import dataclasses
import datetime
import decimal


@dataclasses.dataclass(frozen=True)
class Transaction:
    extractor: str
    # the filename of import source
    file: str | None = None
    # the entry line number of the source file
    lineno: int | None = None
    # the entry line number of the source file in reverse order. comes handy for CSV files in desc datetime order
    reversed_lineno: int | None = None
    # date of the transaction
    date: datetime.date | None = None
    # date when the transaction posted
    post_date: datetime.date | None = None
    # timestamp of the transaction
    timestamp: datetime.datetime | None = None
    # timezone of the transaction, needs to be one of timezone value supported by pytz
    timezone: str | None = None
    # description of the transaction
    desc: str | None = None
    # description of the transaction provided by the bank
    bank_desc: str | None = None
    # transaction amount
    amount: decimal.Decimal | None = None
    # ISO 4217 currency symbol
    currency: str | None = None
    # category of the transaction, like Entertainment, Shopping, etc..
    category: str | None = None
    # status of the transaction
    status: str | None = None
    # type of the transaction, such as Sale, Return, Debit, etc
    type: str | None = None
    # Source account of the transaction
    source_account: str | None = None
    # destination account of the transaction
    dest_account: str | None = None
    # note or memo for the transaction
    note: str | None = None
    # Reference value
    reference: str | None = None
    # Payee of the transaction
    payee: str | None = None



import csv
import datetime
import decimal
import typing

import pytz

from ..data_types import Transaction


def parse_date(date_str: str) -> datetime.date:
    parts = date_str.split("-")
    return datetime.date(int(parts[-1]), *(map(int, parts[:-1])))


def parse_time(time_str: str) -> datetime.time:
    parts = time_str.split(":")
    return datetime.time(*(map(int, parts)))


def parse_datetime(timestamp_str: str) -> datetime.datetime:
    parts = timestamp_str.split(" ")
    date = parse_date(parts[0])
    time = parse_time(parts[1])
    return datetime.datetime.combine(date, time)


class MercuryExtractor:
    EXTRACTOR_NAME = "mercury"
    DEFAULT_IMPORT_ID = "{{ file }}:{{ reversed_lineno }}"

    def __init__(self, input_file: typing.TextIO):
        self.input_file = input_file

    def __call__(self) -> typing.Generator[Transaction, None, None]:
        filename = None
        if hasattr(self.input_file, "name"):
            filename = self.input_file.name
        reader = csv.DictReader(self.input_file)
        # this consumes quite some memory, but it should be fine assume most csv dump files are small
        rows = list(reader)
        timezone = pytz.UTC
        for i, row in enumerate(rows):
            yield Transaction(
                extractor=self.EXTRACTOR_NAME,
                file=filename,
                lineno=i + 1,
                reversed_lineno=len(rows) - i,
                date=parse_date(row["Date (UTC)"]),
                desc=row["Description"],
                amount=decimal.Decimal(row["Amount"]),
                status=row["Status"],
                source_account=row["Source Account"],
                bank_desc=row["Bank Description"],
                reference=row["Reference"],
                note=row["Note"],
                category=row["Category"],
                currency=row["Original Currency"],
                timestamp=parse_datetime(row["Timestamp"]).replace(tzinfo=timezone),
                timezone="UTC",
            )
